@@ -22,6 +22,7 @@ CanDo view components.
 import pytz
 from xml.sax.saxutils import quoteattr
 
+import zope.lifecycleevent
 from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
 from zope.cachedescriptors.property import Lazy
 from zope.component import queryMultiAdapter
@@ -102,6 +103,7 @@ from schooltool.cando.model import getNodeCatalog
 from schooltool.cando.model import getOrderedByHierarchy
 from schooltool.cando.skill import getSkillCatalog, getSkillSetCatalog
 from schooltool.cando.skill import querySkillScoreSystem
+from schooltool.cando.skill import searchable_common_indexes
 from schooltool.cando.browser.skill import SkillAddView
 from schooltool.cando import CanDoMessage as _
 
@@ -945,6 +947,10 @@ class AggregateNodesTableFilter(schooltool.table.ajax.IndexedTableFilter):
     def search_title_id(self):
         return self.manager.html_id+'-title'
 
+    @property
+    def search_index_id(self):
+        return self.manager.html_id+'-index'
+
     @property
     def search_layer_ids(self):
         return self.manager.html_id+"-layers"
@@ -995,10 +1001,16 @@ class AggregateNodesTableFilter(schooltool.table.ajax.IndexedTableFilter):
         found_ids = set()
         query = buildQueryString(self.request.get('SEARCH', ''))
 
+        searchable_index_id = 'text'
+        if self.search_index_id in self.request:
+            index_id = self.request[self.search_index_id]
+            if index_id in searchable_common_indexes:
+                searchable_index_id = index_id
+
         if self.skill_layer_id in request_layer_ids:
             catalog = getSkillCatalog()
             if query:
-                index = catalog['text']
+                index = catalog[searchable_index_id]
                 found_in_catalog = index.apply(query)
             else:
                 found_in_catalog = tuple(catalog.extent)
@@ -1009,7 +1021,7 @@ class AggregateNodesTableFilter(schooltool.table.ajax.IndexedTableFilter):
         if self.skillset_layer_id in request_layer_ids:
             catalog = getSkillSetCatalog()
             if query:
-                index = catalog['text']
+                index = catalog[searchable_index_id]
                 found_in_catalog = index.apply(query)
             else:
                 found_in_catalog = tuple(catalog.extent)
@@ -1019,7 +1031,7 @@ class AggregateNodesTableFilter(schooltool.table.ajax.IndexedTableFilter):
 
         catalog = getNodeCatalog()
         if query:
-            index = catalog['text']
+            index = catalog[searchable_index_id]
             found_in_catalog = set(index.apply(query))
         else:
             found_in_catalog = set(catalog.extent)
@@ -1044,6 +1056,21 @@ class AggregateNodesTableFilter(schooltool.table.ajax.IndexedTableFilter):
         retired = index.values_to_documents.get(True, set())
         return set(found_ids).difference(retired)
 
+    @Lazy
+    def search_indexes(self):
+        order = (
+            ('text_ID', _('ID')),
+            ('text_label', _('Label')),
+            ('text_title', _('Title')),
+            )
+        result = []
+        for index_id, title in order:
+            result.append({
+                    'id': index_id,
+                    'title': title,
+                    })
+        return result
+
 
 class AggregateNodesSkillsSearchTable(table.ajax.IndexedTable):
 
@@ -1162,10 +1189,153 @@ class NodesSearchTable(AggregateNodesSkillsSearchTable):
         return [label, title, layers]
 
 
+class RetireNodeColumn(table.column.CheckboxColumn):
+
+    tokens_name = None
+
+    def __init__(self, *args, **kw):
+        if 'tokens_name' in kw:
+            self.tokens_name = kw.pop('tokens_name')
+        else:
+            prefix = kw.get('prefix')
+            self.tokens_name = '.'.join(filter(None, ["displayed", prefix, "tokens"]))
+        table.column.CheckboxColumn.__init__(self, *args, **kw)
+
+
+    def renderHeader(self, formatter):
+        cell = table.column.CheckboxColumn.renderHeader(self, formatter)
+        cell += ('<input type="checkbox" '
+                 'onclick="return ST.table.check_children(this);" />')
+        return cell
+
+    def renderCell(self, item, formatter):
+        cell = table.column.CheckboxColumn.renderCell(self, item, formatter)
+        cell += ('\n<input name="%(tokens_name)s" value="%(tokens_value)s" '
+                 'type="hidden" />') % (
+            {'tokens_name': self.tokens_name,
+             'tokens_value': self.id_getter(item)})
+        return cell
+
+
+class RetireNodesTable(NodesSearchTable):
+
+    display_success_dialog = 'DISPLAY_SUCCESS_DIALOG'
+
+    def columns(self):
+        columns = NodesSearchTable.columns(self)
+        int_ids = getUtility(IIntIds)
+        columns.insert(0, RetireNodeColumn(
+                prefix="active", name="active",
+                title='',
+                id_getter=lambda node: str(int_ids.getId(node)),
+                value_getter=lambda node: node.retired))
+        return columns
+
+    @property
+    def success_dialog_title(self):
+        return _('Changes saved')
+
+    @property
+    def success_dialog_url(self):
+        return '%s/retire_successful.html' % absoluteURL(self.context,
+                                                         self.request)
+
+
+class RetireNodesScript(flourish.viewlet.Viewlet):
+
+    template = ViewPageTemplateFile('templates/retire_nodes_script.pt')
+
+    def render(self, *args, **kw):
+        if self.manager.display_success_dialog in self.request:
+            return self.template(*args, **kw)
+        return ''
+
+
+class RetireNodesInstructionViewlet(flourish.viewlet.Viewlet):
+
+    template = InlineViewPageTemplate('''
+      <h3 i18n:domain="schooltool.cando" i18n:translate="">
+        Select items to deprecate:
+      </h3>
+    ''')
+
+    def render(self, *args, **kw):
+        if not self.manager._items:
+            return ''
+        return self.template(*args, **kw)
+
+
 class NodesSearchTableFilter(AggregateNodesTableFilter):
     pass
 
 
+class RetireNodesTableFilter(AggregateNodesTableFilter):
+
+    def removeRetired(self, catalog, found_ids):
+        return set(found_ids)
+
+
+class SaveRetiredButton(flourish.viewlet.Viewlet):
+
+    template = flourish.templates.File('templates/retire_nodes_save_button.pt')
+
+    button_name = 'SAVE_RESULTS'
+    cancel_name = 'CANCEL'
+    token_key = 'displayed.active.tokens'
+    checkbox_key = 'active.'
+
+    @property
+    def onclick(self):
+        return "$('#%(button_id)s').attr('disabled', 'disabled'); "\
+               "return ST.table.on_form_submit('%(container_id)s', '#%(button_id)s')" % ({
+                'container_id': self.manager.html_id,
+                'button_id': self.html_id,
+                })
+
+    @property
+    def html_id(self):
+        return '-'.join(filter(None, [self.manager.html_id, 'save_button']))
+
+    def saveChanges(self):
+        changed = False
+        checkbox_key = self.checkbox_key
+        checked_ids = [iid[len(checkbox_key):]
+                       for iid in self.request
+                       if (iid.startswith(checkbox_key) and
+                           self.request[iid])]
+        displayed_ids = self.request.get(self.token_key, [])
+        if not isinstance(displayed_ids, list):
+            displayed_ids = [displayed_ids]
+        int_ids = getUtility(IIntIds)
+        for sid in displayed_ids:
+            retired = sid in checked_ids
+            try:
+                iid = int(sid)
+            except (TypeError, ValueError):
+                continue
+            node = int_ids.queryObject(iid)
+            if node is None:
+                continue
+            if node.retired != retired:
+                node.retired = retired
+                changed = True
+                zope.lifecycleevent.modified(node)
+        return changed
+
+    def update(self):
+        if self.button_name in self.request:
+            changed = self.saveChanges()
+            if changed:
+                self.request.form[self.manager.display_success_dialog] = True
+        if self.cancel_name in self.request:
+            self.request.response.redirect(self.nextURL())
+
+    def nextURL(self):
+        app = ISchoolToolApplication(None)
+        container = IDocumentContainer(app)
+        return absoluteURL(container, self.request)
+
+
 class NodeChildrenTable(SkillSearchTable):
 
     batch_size = 0
@@ -1421,7 +1591,7 @@ class CanDoGradeStudentTableFormatter(table.ajax.AJAXFormSortFormatter):
 def label_title_formatter(obj, item, formatter):
     title = obj.title
     label = getattr(obj, 'label')
-    if label is not None:
+    if label:
         title = '%s: %s' % (label, title)
     return title
 
@@ -1432,6 +1602,18 @@ def skill_score_formatter(score, item, formatter):
     return ''
 
 
+def get_worksheets(student_gradebook):
+    skills_gradebook = student_gradebook.gradebook
+    section = ISection(proxy.removeSecurityProxy(skills_gradebook))
+    return ISectionSkills(section)
+
+
+def get_projects(student_gradebook):
+    project_gradebook = student_gradebook.gradebook
+    section = ISection(proxy.removeSecurityProxy(project_gradebook))
+    return IProjects(section)
+
+
 class CanDoGradeStudentTableBase(table.ajax.Table):
 
     batch_size = 0
@@ -1441,12 +1623,18 @@ class CanDoGradeStudentTableBase(table.ajax.Table):
         skillset = skill.__parent__
         return '%s.%s' % (skillset.__name__, skill.__name__)
 
+    @property
+    def worksheets(self):
+        if self.view.isSkillsGradebook:
+            return get_worksheets(self.context)
+        else:
+            return get_projects(self.context)
+
     def items(self):
         result = []
         iep = IStudentIEP(self.context.student)
         iep_skills = iep.getIEPSkills(self.view.gradebook.section)
-        worksheets = self.context.__parent__.__parent__.__parent__
-        for worksheet in worksheets.values():
+        for worksheet in self.worksheets.values():
             if self.view.isSkillsGradebook:
                 gradebook = ISkillsGradebook(worksheet)
                 skillset_label = worksheet.label
@@ -1783,7 +1971,6 @@ class CanDoStudentGradebookReportTask(ReportTask):
         gradebook = ISkillsGradebook(worksheet)
         student_gradebook = getMultiAdapter((student, gradebook),
                                             ICanDoStudentGradebook)
-        student_gradebook.__parent__ = gradebook
         return student_gradebook
 
     @context.setter
@@ -1825,10 +2012,9 @@ class RMLSkillColumn(table.pdf.RMLGetterColumn):
             self.style.para_class, self.escape(value))
 
 
-def getScoreSystems(gradebook):
+def getScoreSystems(student_gradebook):
     result = {}
-    worksheet = gradebook.__parent__.__parent__
-    worksheets = worksheet.__parent__
+    worksheets = get_worksheets(student_gradebook)
     for worksheet in worksheets.values():
         for activity in worksheet.values():
             scoresystem = proxy.removeSecurityProxy(activity.scoresystem)
@@ -2048,7 +2234,6 @@ class StudentCompetencySectionReportPDFStory(flourish.report.PDFStory):
                               cmp=collator.cmp):
             student_gradebook = getMultiAdapter((student, gradebook),
                                                 ICanDoStudentGradebook)
-            student_gradebook.__parent__ = gradebook
             student_scr = getMultiAdapter(
                 (student_gradebook, self.request, self.view, self),
                 name="student_scr")
